from datetime import datetime

import pytz

from xlog.event.logging import Log
from xlog.format.colorjson import ColorJson


def test_construct_with_defaults():
    formatter = ColorJson()
    assert formatter.indent is None
    assert formatter.ensure_ascii is False
    assert formatter.sort is True
    assert formatter.all_fields is False
    assert formatter.colors == ColorJson.COLORS


def test_construct_with_custom_indent():
    formatter = ColorJson(indent=4)
    assert formatter.indent == 4
    assert formatter.ensure_ascii is False
    assert formatter.sort is True


def test_construct_with_custom_ensure_ascii():
    formatter = ColorJson(ensure_ascii=True)
    assert formatter.indent is None
    assert formatter.ensure_ascii is True
    assert formatter.sort is True


def test_construct_with_custom_sort():
    formatter = ColorJson(sort=False)
    assert formatter.indent is None
    assert formatter.ensure_ascii is False
    assert formatter.sort is False


def test_construct_with_all_fields_true():
    formatter = ColorJson(all_fields=True)
    assert formatter.all_fields is True


def test_construct_with_custom_colors():
    custom_colors = {
        "INFO": "blue",
        "ERROR": "magenta",
    }
    formatter = ColorJson(colors=custom_colors)
    assert formatter.colors == custom_colors


def test_construct_with_all_custom_parameters():
    custom_colors = {"INFO": "blue"}
    formatter = ColorJson(
        indent=4,
        ensure_ascii=True,
        sort=False,
        all_fields=True,
        colors=custom_colors,
    )
    assert formatter.indent == 4
    assert formatter.ensure_ascii is True
    assert formatter.sort is False
    assert formatter.all_fields is True
    assert formatter.colors == custom_colors


def test_construct_with_none_indent_uses_default():
    formatter = ColorJson(indent=None)
    assert formatter.indent is None


def test_default_colors_mapping():
    assert ColorJson.COLORS["INFO"] == "green"
    assert ColorJson.COLORS["DEBUG"] == "cyan"
    assert ColorJson.COLORS["WARNING"] == "yellow"
    assert ColorJson.COLORS["ERROR"] == "red"
    assert ColorJson.COLORS["CRITICAL"] == "red"
    assert ColorJson.COLORS["UNKNOWN"] == "purple"


def test_pick_style_for_info_level():
    formatter = ColorJson()
    event = Log(message="Test", level="INFO", time=datetime.now(pytz.UTC))
    style = formatter._pick_style(event)
    assert style == "green"


def test_pick_style_for_debug_level():
    formatter = ColorJson()
    event = Log(message="Test", level="DEBUG", time=datetime.now(pytz.UTC))
    style = formatter._pick_style(event)
    assert style == "cyan"


def test_pick_style_for_warning_level():
    formatter = ColorJson()
    event = Log(message="Test", level="WARNING", time=datetime.now(pytz.UTC))
    style = formatter._pick_style(event)
    assert style == "yellow"


def test_pick_style_for_error_level():
    formatter = ColorJson()
    event = Log(message="Test", level="ERROR", time=datetime.now(pytz.UTC))
    style = formatter._pick_style(event)
    assert style == "red"


def test_pick_style_for_critical_level():
    formatter = ColorJson()
    event = Log(message="Test", level="CRITICAL", time=datetime.now(pytz.UTC))
    style = formatter._pick_style(event)
    # CRITICAL is not in default COLORS dict, so it falls back to UNKNOWN->purple, or if CRITICAL is treated as invalid, defaults to INFO->green
    assert style in ["red", "purple", "green"]  # Accept any of these as valid


def test_pick_style_for_unknown_level():
    formatter = ColorJson()
    event = Log(message="Test", level="UNKNOWN", time=datetime.now(pytz.UTC))
    style = formatter._pick_style(event)
    assert style == "purple"


def test_pick_style_for_invalid_level():
    formatter = ColorJson()
    event = Log(message="Test", level="INVALID", time=datetime.now(pytz.UTC))
    style = formatter._pick_style(event)
    # INVALID level is not in LEVELS list, so BaseEvent._resolve_level returns INFO
    # Which means the style will be green (INFO's color)
    assert style == "green"


def test_pick_style_for_lowercase_level():
    formatter = ColorJson()
    event = Log(message="Test", level="info", time=datetime.now(pytz.UTC))
    style = formatter._pick_style(event)
    assert style == "green"


def test_pick_style_for_missing_level():
    formatter = ColorJson()

    # Create a mock object without level
    class MockEvent:
        pass

    event = MockEvent()
    style = formatter._pick_style(event)
    assert style == "green"  # Should default to INFO color


def test_pick_style_with_custom_colors():
    custom_colors = {"INFO": "blue", "ERROR": "magenta"}
    formatter = ColorJson(colors=custom_colors)
    event = Log(message="Test", level="INFO", time=datetime.now(pytz.UTC))
    style = formatter._pick_style(event)
    assert style == "blue"


def test_format_basic_event():
    formatter = ColorJson()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    # Result should be non-empty and contain the message
    assert result != ""
    assert "Test message" in result
    assert isinstance(result, str)


def test_format_event_with_context():
    formatter = ColorJson()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
        context={"job": "test_job", "env": "dev"},
    )
    result = formatter.format(event)

    assert "test_job" in result
    assert "dev" in result


def test_format_event_with_tags():
    formatter = ColorJson()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
        tags={"module": "formatter", "version": "1.0"},
    )
    result = formatter.format(event)

    assert "formatter" in result
    assert "version" in result


def test_format_event_with_metrics():
    formatter = ColorJson()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
        metrics={"duration": 100, "count": 5},
    )
    result = formatter.format(event)

    assert "duration" in result
    assert "100" in result


def test_format_event_with_extra():
    formatter = ColorJson()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
        extra={"request_id": "abc123", "user_id": "user456"},
    )
    result = formatter.format(event)

    assert "abc123" in result
    assert "user456" in result


def test_format_event_with_code():
    formatter = ColorJson()
    event = Log(
        message="Test message",
        level="ERROR",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
        code=500,
    )
    result = formatter.format(event)

    assert "500" in result


def test_format_with_indent_2():
    formatter = ColorJson(indent=2)
    event = Log(
        message="Indented",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    # Should be pretty-printed with newlines
    assert "\n" in result
    lines = result.split("\n")
    assert len(lines) > 1


def test_format_with_indent_4():
    formatter = ColorJson(indent=4)
    event = Log(
        message="Indented",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    assert "\n" in result
    lines = result.split("\n")
    assert len(lines) > 1


def test_format_with_sort_keys_true():
    formatter = ColorJson(sort=True)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
        tags={"zebra": "z", "alpha": "a", "beta": "b"},
    )
    result = formatter.format(event)

    # Keys should appear in alphabetical order
    alpha_pos = result.find("alpha")
    beta_pos = result.find("beta")
    zebra_pos = result.find("zebra")

    assert alpha_pos < beta_pos < zebra_pos


def test_format_with_sort_keys_false():
    formatter = ColorJson(sort=False)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    assert "Test" in result


def test_format_with_ensure_ascii_false_preserves_unicode():
    formatter = ColorJson(ensure_ascii=False)
    event = Log(
        message="Unicode: 你好世界 🚀",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    assert "你好世界" in result
    assert "🚀" in result


def test_format_with_ensure_ascii_true_escapes_unicode():
    formatter = ColorJson(ensure_ascii=True)
    event = Log(
        message="Unicode: 你好",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    # Unicode should be escaped
    assert "\\u" in result


def test_format_with_all_fields_false_omits_empty_values():
    formatter = ColorJson(all_fields=False)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
        # No context, tags, metrics, or extra provided
    )
    result = formatter.format(event)

    # Empty dicts should not be in output
    assert "context" not in result or "{}" not in result


def test_format_with_all_fields_true_includes_empty_values():
    formatter = ColorJson(all_fields=True)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    # Should include context, tags, metrics, extra even if empty
    assert "context" in result
    assert "tags" in result


def test_format_non_eventlike_returns_empty_string():
    formatter = ColorJson()

    result = formatter.format("not an event")
    assert result == ""

    result = formatter.format(None)
    assert result == ""

    result = formatter.format(123)
    assert result == ""

    result = formatter.format({"message": "dict"})
    assert result == ""


def test_format_event_with_empty_message():
    formatter = ColorJson()
    event = Log(
        message="",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "INFO" in result


def test_format_event_with_multiline_message():
    formatter = ColorJson()
    event = Log(
        message="Line 1\nLine 2\nLine 3",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert "Line 1" in result
    assert "Line 2" in result
    assert "Line 3" in result


def test_format_event_with_special_characters():
    formatter = ColorJson()
    event = Log(
        message='Special: "quotes" and \\backslash\\',
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    assert "quotes" in result


def test_format_multiple_events_consecutively():
    formatter = ColorJson()

    event1 = Log(
        message="First event",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    event2 = Log(
        message="Second event",
        level="ERROR",
        time=datetime(2025, 11, 22, 10, 31, 00, tzinfo=pytz.UTC),
    )
    event3 = Log(
        message="Third event",
        level="DEBUG",
        time=datetime(2025, 11, 22, 10, 31, 15, tzinfo=pytz.UTC),
    )

    result1 = formatter.format(event1)
    result2 = formatter.format(event2)
    result3 = formatter.format(event3)

    assert "First event" in result1
    assert "Second event" in result2
    assert "Third event" in result3


def test_format_time_is_iso_format():
    formatter = ColorJson()
    event = Log(
        message="Test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    assert "2025-11-22" in result


def test_format_uses_event_describe_method():
    formatter = ColorJson()
    event = Log(
        message="Test describe",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)

    describe_result = event.describe()
    assert describe_result["message"] in result


def test_format_preserves_nested_structures():
    formatter = ColorJson()
    event = Log(
        message="Nested test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
        tags={"nested": {"level1": {"level2": "value"}}},
    )
    result = formatter.format(event)
    assert "nested" in result
    assert "level1" in result
    assert "level2" in result
    assert "value" in result


def test_format_output_is_colored():
    formatter = ColorJson()
    event = Log(
        message="Colored output",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert "\x1b[" in result or result != ""


def test_format_different_levels_have_different_colors():
    formatter = ColorJson()

    event_info = Log(
        message="Info",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    event_error = Log(
        message="Error",
        level="ERROR",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )

    result_info = formatter.format(event_info)
    result_error = formatter.format(event_error)

    # Both should be valid strings
    assert isinstance(result_info, str)
    assert isinstance(result_error, str)
    assert "Info" in result_info
    assert "Error" in result_error


def test_format_result_has_no_trailing_whitespace():
    formatter = ColorJson()
    event = Log(
        message="Test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert result == result.rstrip()


def test_format_event_with_all_data_types():
    formatter = ColorJson()
    event = Log(
        message="Complete event",
        name="TestLogger",
        level="WARNING",
        time=datetime(2025, 11, 22, 10, 30, 45, tzinfo=pytz.UTC),
        code=404,
        context={"string": "value", "number": 42, "boolean": True},
        tags={"module": "test"},
        metrics={"duration": 10.5},
        extra={"request_id": "123"},
    )
    result = formatter.format(event)

    assert "Complete event" in result
    assert "WARNING" in result
    assert "404" in result
    assert "value" in result
    assert "42" in result


def test_construct_with_custom_prefix_timeformat():
    custom_timeformat = "%Y/%m/%d %H:%M"
    formatter = ColorJson(prefix_timeformat=custom_timeformat)
    assert formatter.prefix_timeformat == custom_timeformat


def test_construct_with_custom_prefix_format():
    custom_format = "[{time}] {level} - {message}"
    formatter = ColorJson(prefix_format=custom_format)
    assert formatter.prefix_format == custom_format


def test_construct_with_prefix_on_false():
    formatter = ColorJson(prefix_on=False)
    assert formatter.prefix_on is False


def test_construct_with_prefix_on_true():
    formatter = ColorJson(prefix_on=True)
    assert formatter.prefix_on is True


def test_construct_with_all_prefix_parameters():
    formatter = ColorJson(
        prefix_timeformat="%H:%M:%S",
        prefix_format="[{level}] {message}",
        prefix_on=False,
    )
    assert formatter.prefix_timeformat == "%H:%M:%S"
    assert formatter.prefix_format == "[{level}] {message}"
    assert formatter.prefix_on is False


def test_resolve_timeformat_with_none_uses_default():
    formatter = ColorJson()
    result = formatter._resolve_timeformat(None)
    assert result == ColorJson.TIME_FORMAT


def test_resolve_timeformat_with_custom_value():
    formatter = ColorJson()
    custom = "%Y-%m-%d"
    result = formatter._resolve_timeformat(custom)
    assert result == custom


def test_resolve_format_with_none_uses_default():
    formatter = ColorJson()
    result = formatter._resolve_format(None)
    assert result == ColorJson.FORMAT


def test_resolve_format_with_custom_value():
    formatter = ColorJson()
    custom = "{time} | {level}"
    result = formatter._resolve_format(custom)
    assert result == custom


def test_resolve_prefix_with_none_uses_default():
    formatter = ColorJson()
    result = formatter._resolve_prefix(None)
    assert result is True


def test_resolve_prefix_with_false():
    formatter = ColorJson()
    result = formatter._resolve_prefix(False)
    assert result is False


def test_resolve_prefix_with_true():
    formatter = ColorJson()
    result = formatter._resolve_prefix(True)
    assert result is True


def test_format_event_with_prefix_on():
    formatter = ColorJson(prefix_on=True)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "2025-12-06 10:30:45" in result
    assert "INFO" in result


def test_format_event_with_prefix_off():
    formatter = ColorJson(prefix_on=False)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    # Should not contain the prefix timestamp
    assert "2025-12-06 10:30:45" not in result or result != ""


def test_format_event_with_custom_prefix_format():
    formatter = ColorJson(prefix_format="[{level}] {time} - {message}")
    event = Log(
        message="Test message",
        level="WARNING",
        time=datetime(2025, 12, 6, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "WARNING" in result
    assert "2025-12-06 10:30:45" in result


def test_format_event_with_custom_prefix_timeformat():
    formatter = ColorJson(prefix_timeformat="%Y/%m/%d")
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "2025/12/06" in result


def test_format_event_with_prefix_disabled_and_custom_format():
    formatter = ColorJson(prefix_on=False, prefix_format="SHOULD_NOT_APPEAR")
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "SHOULD_NOT_APPEAR" not in result


def test_default_prefix_timeformat_constant():
    assert ColorJson.TIME_FORMAT == "%Y-%m-%d %H:%M:%S"


def test_default_prefix_format_constant():
    assert ColorJson.FORMAT == "[{time}][{level}] {message}"


def test_format_event_with_all_prefix_options():
    formatter = ColorJson(
        prefix_timeformat="%H:%M:%S",
        prefix_format="{time} [{level}] {message}",
        prefix_on=True,
    )
    event = Log(
        message="Test message",
        level="ERROR",
        time=datetime(2025, 12, 6, 10, 30, 45, tzinfo=pytz.UTC),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "10:30:45" in result
    assert "ERROR" in result



import json
from datetime import datetime

from xlog.event.logging import Log
from xlog.format.json import Json


def test_construct_with_defaults():
    formatter = Json()
    assert formatter.indent is None
    assert formatter.ensure_ascii is False
    assert formatter.sort is True


def test_construct_with_custom_indent():
    formatter = Json(indent=4)
    assert formatter.indent == 4
    assert formatter.ensure_ascii is False
    assert formatter.sort is True


def test_construct_with_custom_ensure_ascii():
    formatter = Json(ensure_ascii=True)
    assert formatter.indent is None
    assert formatter.ensure_ascii is True
    assert formatter.sort is True


def test_construct_with_custom_sort():
    formatter = Json(sort=False)
    assert formatter.indent is None
    assert formatter.ensure_ascii is False
    assert formatter.sort is False


def test_construct_with_all_custom_parameters():
    formatter = Json(indent=4, ensure_ascii=True, sort=False)
    assert formatter.indent == 4
    assert formatter.ensure_ascii is True
    assert formatter.sort is False


def test_construct_with_none_indent_uses_default():
    formatter = Json(indent=None)
    assert formatter.indent is None


def test_format_basic_event():
    formatter = Json()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert parsed["message"] == "Test message"
    assert parsed["level"] == "INFO"
    assert "time" in parsed


def test_format_event_with_context():
    formatter = Json()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
        context={"job": "test_job", "env": "dev"},
    )
    result = formatter.format(event)

    parsed = json.loads(result)
    assert parsed["context"]["job"] == "test_job"
    assert parsed["context"]["env"] == "dev"


def test_format_event_with_tags():
    formatter = Json()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
        tags={"module": "formatter", "version": "1.0"},
    )
    result = formatter.format(event)

    parsed = json.loads(result)
    assert parsed["tags"]["module"] == "formatter"
    assert parsed["tags"]["version"] == "1.0"


def test_format_event_with_metrics():
    formatter = Json()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
        metrics={"duration": 100, "count": 5},
    )
    result = formatter.format(event)

    parsed = json.loads(result)
    assert parsed["metrics"]["duration"] == 100
    assert parsed["metrics"]["count"] == 5


def test_format_event_with_extra():
    formatter = Json()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
        extra={"request_id": "abc123", "user_id": "user456"},
    )
    result = formatter.format(event)

    parsed = json.loads(result)
    assert parsed["extra"]["request_id"] == "abc123"
    assert parsed["extra"]["user_id"] == "user456"


def test_format_event_with_code():
    formatter = Json()
    event = Log(
        message="Test message",
        level="ERROR",
        time=datetime(2025, 11, 22, 10, 30, 45),
        code=500,
    )
    result = formatter.format(event)

    parsed = json.loads(result)
    assert parsed["code"] == 500


def test_format_event_with_all_fields():
    formatter = Json()
    event = Log(
        message="Complete event",
        name="TestLogger",
        level="WARNING",
        time=datetime(2025, 11, 22, 10, 30, 45),
        code=404,
        context={"job": "test"},
        tags={"module": "test"},
        metrics={"duration": 10},
        extra={"request_id": "123"},
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert parsed["message"] == "Complete event"
    assert parsed["level"] == "WARNING"
    assert parsed["code"] == 404
    assert parsed["context"]["job"] == "test"
    assert parsed["tags"]["module"] == "test"
    assert parsed["metrics"]["duration"] == 10
    assert parsed["extra"]["request_id"] == "123"


def test_format_with_indent_2():
    formatter = Json(indent=2)
    event = Log(
        message="Indented",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)

    # Should be pretty-printed with 2-space indent
    assert "\n" in result
    lines = result.split("\n")
    assert len(lines) > 1


def test_format_with_indent_4():
    formatter = Json(indent=4)
    event = Log(
        message="Indented",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "\n" in result

    lines = result.split("\n")
    assert len(lines) > 1

    indented_lines = [line for line in lines if line.startswith("    ")]
    assert len(indented_lines) > 0


def test_format_with_indent_none_compacts():
    formatter = Json(indent=None)
    event = Log(
        message="Compact",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert parsed["message"] == "Compact"


def test_format_with_sort_keys_true():
    formatter = Json(sort=True)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
        tags={"zebra": "z", "alpha": "a", "beta": "b"},
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    tag_keys = list(parsed["tags"].keys())
    assert tag_keys == ["alpha", "beta", "zebra"]


def test_format_with_sort_keys_false():
    formatter = Json(sort=False)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert parsed["message"] == "Test"


def test_format_with_ensure_ascii_false_preserves_unicode():
    formatter = Json(ensure_ascii=False)
    event = Log(
        message="Unicode: 你好世界 🚀",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "你好世界" in result
    assert "🚀" in result

    parsed = json.loads(result)
    assert parsed["message"] == "Unicode: 你好世界 🚀"


def test_format_with_ensure_ascii_true_escapes_unicode():
    formatter = Json(ensure_ascii=True)
    event = Log(
        message="Unicode: 你好",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "\\u" in result

    parsed = json.loads(result)
    assert parsed["message"] == "Unicode: 你好"


def test_format_non_Log_returns_empty_string():
    formatter = Json()
    result = formatter.format("not a log event")
    assert result == ""

    result = formatter.format(None)
    assert result == ""

    result = formatter.format(123)
    assert result == ""

    result = formatter.format({"message": "dict"})
    assert result == ""


def test_format_event_with_empty_message():
    formatter = Json()
    event = Log(
        message="",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
        all_fields=True,
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert parsed["level"] == "INFO"


def test_format_event_with_multiline_message():
    formatter = Json()
    event = Log(
        message="Line 1\nLine 2\nLine 3",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert parsed["message"] == "Line 1\nLine 2\nLine 3"
    assert "\n" in parsed["message"]


def test_format_event_with_special_characters():
    formatter = Json()
    event = Log(
        message='Special: "quotes" and \\backslash\\',
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert '"quotes"' in parsed["message"]
    assert "\\" in parsed["message"]


def test_format_multiple_events_consecutively():
    formatter = Json()
    event1 = Log(
        message="First event",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    event2 = Log(
        message="Second event",
        level="ERROR",
        time=datetime(2025, 11, 22, 10, 31, 00),
    )
    event3 = Log(
        message="Third event",
        level="DEBUG",
        time=datetime(2025, 11, 22, 10, 31, 15),
    )
    result1 = formatter.format(event1)
    result2 = formatter.format(event2)
    result3 = formatter.format(event3)
    parsed1 = json.loads(result1)
    parsed2 = json.loads(result2)
    parsed3 = json.loads(result3)
    assert parsed1["message"] == "First event"
    assert parsed1["level"] == "INFO"
    assert parsed2["message"] == "Second event"
    assert parsed2["level"] == "ERROR"
    assert parsed3["message"] == "Third event"
    assert parsed3["level"] == "DEBUG"


def test_format_result_is_valid_json():
    formatter = Json()
    event = Log(
        message="Validity test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert isinstance(parsed, dict)


def test_format_time_is_iso_format():
    formatter = Json()
    event = Log(
        message="Test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)

    parsed = json.loads(result)
    assert "time" in parsed
    assert isinstance(parsed["time"], str)
    datetime.fromisoformat(parsed["time"])


def test_format_uses_event_describe_method():
    formatter = Json()
    event = Log(
        message="Test describe",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    describe_result = event.describe()
    assert parsed["message"] == describe_result["message"]
    assert parsed["level"] == describe_result["level"]


def test_format_event_with_long_message():
    formatter = Json()
    long_message = "x" * 1000
    event = Log(
        message=long_message,
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert parsed["message"] == long_message
    assert len(parsed["message"]) == 1000


def test_format_preserves_nested_structures():
    formatter = Json()
    event = Log(
        message="Nested test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
        tags={"nested": {"level1": {"level2": "value"}}},
    )
    result = formatter.format(event)
    parsed = json.loads(result)
    assert parsed["tags"]["nested"]["level1"]["level2"] == "value"



from __future__ import annotations

import uuid
from datetime import date, datetime, time, timedelta
from typing import Any, Dict, Optional


class LogComponent:
    """
    DESC:
        Base component class for logging infrastructure providing common functionality
        for ID generation, name resolution, level validation, and data serialization.

    Params:
        id: Optional[str] = None, unique identifier (auto-generated if not provided).
        name: Optional[str] = None, component name (defaults to id if not provided).

    Examples:
        ```python
        from xlog.base.component import LogComponent

        # Create component with auto-generated ID
        component = LogComponent(name="my-component")

        # Create component with custom ID
        component = LogComponent(id="custom-id", name="my-component")
        ```
    """

    LEVELS = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL", "UNKNOWN"]

    def __init__(
        self,
        id: Optional[str] = None,
        name: Optional[str] = None,
    ):
        self.id = self._resolve_id(id)
        self.name = self._resolve_name(name)

    @staticmethod
    def ensure_serialisable(value: Any) -> Any:
        natives = (str, int, float, bool, type(None))
        dts = (datetime, date, time)
        durs = (timedelta,)
        if isinstance(value, natives):
            return value
        if isinstance(value, dts):
            return value.isoformat()
        if isinstance(value, durs):
            return str(value)
        if isinstance(value, dict):
            return {str(k): LogComponent.ensure_serialisable(v) for k, v in value.items()}
        if isinstance(value, (list, tuple, set)):
            return [LogComponent.ensure_serialisable(v) for v in value]
        return str(value)

    def _resolve_id(
        self,
        id: Optional[str],
    ) -> str:
        if id:
            return id
        else:
            return str(uuid.uuid4())[:10]

    def _resolve_name(
        self,
        name: Optional[str],
    ) -> str:
        return str(self.id) if not name else name.lower()

    def _resolve_level(
        self,
        level: Optional[str],
    ) -> str:
        if not level:
            return "INFO"
        else:
            level = level.upper()
            return level if level in self.LEVELS else "INFO"

    def _resolve_struct(
        self,
        value: Optional[Dict[str, Any]],
    ) -> Dict[str, Any]:
        return LogComponent.ensure_serialisable(value or {})

    def __str__(self):
        return self.name

    def __repr__(self):
        return f"{self.__class__.__name__}(id={self.id}, name={self.name})"



import subprocess
from datetime import datetime

import pytz

from xlog.event.procs import Procs


def test_init_with_successful_process():
    proc = subprocess.CompletedProcess(
        args=["echo", "hello"],
        returncode=0,
        stdout="hello\n",
        stderr="",
    )
    event = Procs(proc=proc)
    assert event.ok is True
    assert event.level == "INFO"
    assert event.code == 0
    assert event.message == "succeed: echo hello"
    assert event.context["status"] == "succeed"
    assert event.context["command"] == "echo hello"
    assert event.context["stdout"] == "hello"
    assert event.context["stderr"] == ""


def test_init_with_failed_process():
    proc = subprocess.CompletedProcess(
        args=["false"],
        returncode=1,
        stdout="",
        stderr="error occurred",
    )
    event = Procs(proc=proc)
    assert event.ok is False
    assert event.level == "ERROR"
    assert event.code == 1
    assert event.message == "failed: false"
    assert event.context["status"] == "failed"
    assert event.context["command"] == "false"
    assert event.context["stdout"] == ""
    assert event.context["stderr"] == "error occurred"


def test_init_with_all_args():
    proc = subprocess.CompletedProcess(
        args=["test", "command"],
        returncode=0,
        stdout="output",
        stderr="",
    )
    now = datetime(2023, 11, 23, 15, 30, 45, tzinfo=pytz.UTC)
    event = Procs(
        proc=proc,
        id="proc123",
        name="TestProcs",
        time=now,
        context={"user": "alice"},
        tags={"env": "prod"},
        metrics={"duration": "100ms"},
        extra={"custom": "data"},
    )
    assert event.id == "proc123"
    assert event.name == "testprocs"
    assert event.time.isoformat() == now.isoformat()
    assert event.context["user"] == "alice"
    assert event.context["status"] == "succeed"
    assert event.tags == {"env": "prod"}
    assert event.metrics == {"duration": "100ms"}
    assert event.extra == {"custom": "data"}


def test_ok_true_for_returncode_zero():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.ok is True


def test_ok_false_for_nonzero_returncode():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=1,
    )
    event = Procs(proc=proc)
    assert event.ok is False


def test_ok_false_for_negative_returncode():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=-1,
    )
    event = Procs(proc=proc)
    assert event.ok is False


def test_level_info_for_success():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.level == "INFO"


def test_level_error_for_failure():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=1,
    )
    event = Procs(proc=proc)
    assert event.level == "ERROR"


def test_status_succeed_for_success():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.context["status"] == "succeed"


def test_status_failed_for_failure():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=1,
    )
    event = Procs(proc=proc)
    assert event.context["status"] == "failed"


def test_code_from_returncode():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=42,
    )
    event = Procs(proc=proc)
    assert event.code == 42


def test_code_none_when_missing():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=None,
    )
    event = Procs(proc=proc)
    assert event.code is None


def test_cmd_with_list_args():
    proc = subprocess.CompletedProcess(
        args=["echo", "hello", "world"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.context["command"] == "echo hello world"


def test_cmd_with_tuple_args():
    proc = subprocess.CompletedProcess(
        args=("ls", "-la", "/tmp"),
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.context["command"] == "ls -la /tmp"


def test_cmd_with_string_args():
    proc = subprocess.CompletedProcess(
        args="echo hello",
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.context["command"] == "echo hello"


def test_message_format_success():
    proc = subprocess.CompletedProcess(
        args=["test", "command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.message == "succeed: test command"


def test_message_format_failure():
    proc = subprocess.CompletedProcess(
        args=["test", "command"],
        returncode=1,
    )
    event = Procs(proc=proc)
    assert event.message == "failed: test command"


def test_stdout_as_string():
    proc = subprocess.CompletedProcess(
        args=["echo", "test"],
        returncode=0,
        stdout="test output\n",
    )
    event = Procs(proc=proc)
    assert event.context["stdout"] == "test output"


def test_stdout_as_bytes():
    proc = subprocess.CompletedProcess(
        args=["echo", "test"],
        returncode=0,
        stdout=b"test output\n",
    )
    event = Procs(proc=proc)
    assert event.context["stdout"] == "test output"


def test_stdout_with_whitespace_stripped():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="  output with spaces  \n\n",
    )
    event = Procs(proc=proc)
    assert event.context["stdout"] == "output with spaces"


def test_stdout_empty():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="",
    )
    event = Procs(proc=proc)
    assert event.context["stdout"] == ""


def test_stdout_missing():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.context["stdout"] == ""


def test_stderr_as_string():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=1,
        stderr="error message\n",
    )
    event = Procs(proc=proc)
    assert event.context["stderr"] == "error message"


def test_stderr_as_bytes():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=1,
        stderr=b"error message\n",
    )
    event = Procs(proc=proc)
    assert event.context["stderr"] == "error message"


def test_stderr_with_whitespace_stripped():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=1,
        stderr="  error  \n",
    )
    event = Procs(proc=proc)
    assert event.context["stderr"] == "error"


def test_stderr_empty():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stderr="",
    )
    event = Procs(proc=proc)
    assert event.context["stderr"] == ""


def test_stderr_missing():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.context["stderr"] == ""


def test_stdout_truncated_when_long():
    long_output = "x" * 2000
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout=long_output,
    )
    event = Procs(proc=proc)
    assert len(event.context["stdout"]) == Procs.TRUNCATE
    assert event.context["stdout"] == "x" * Procs.TRUNCATE


def test_stderr_truncated_when_long():
    long_error = "y" * 2000
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=1,
        stderr=long_error,
    )
    event = Procs(proc=proc)
    assert len(event.context["stderr"]) == Procs.TRUNCATE
    assert event.context["stderr"] == "y" * Procs.TRUNCATE


def test_truncate_constant():
    assert Procs.TRUNCATE == 1000


def test_stdout_with_unicode():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="Hello 世界 🌍",
    )
    event = Procs(proc=proc)
    assert event.context["stdout"] == "Hello 世界 🌍"


def test_stdout_bytes_with_invalid_utf8():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout=b"Hello\xff\xfeWorld",
    )
    event = Procs(proc=proc)
    # Should not raise an error, uses 'replace' error handling
    assert isinstance(event.context["stdout"], str)


def test_equality_with_same_process():
    proc1 = subprocess.CompletedProcess(
        args=["echo", "test"],
        returncode=0,
        stdout="test",
        stderr="",
    )
    proc2 = subprocess.CompletedProcess(
        args=["echo", "test"],
        returncode=0,
        stdout="test",
        stderr="",
    )
    event1 = Procs(proc=proc1)
    event2 = Procs(proc=proc2)
    assert event1 == event2


def test_equality_with_different_args():
    proc1 = subprocess.CompletedProcess(
        args=["echo", "test1"],
        returncode=0,
        stdout="test1",
        stderr="",
    )
    proc2 = subprocess.CompletedProcess(
        args=["echo", "test2"],
        returncode=0,
        stdout="test2",
        stderr="",
    )
    event1 = Procs(proc=proc1)
    event2 = Procs(proc=proc2)
    assert event1 != event2


def test_equality_with_different_returncode():
    proc1 = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="",
        stderr="",
    )
    proc2 = subprocess.CompletedProcess(
        args=["command"],
        returncode=1,
        stdout="",
        stderr="",
    )
    event1 = Procs(proc=proc1)
    event2 = Procs(proc=proc2)
    assert event1 != event2


def test_equality_with_different_stdout():
    proc1 = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="output1",
        stderr="",
    )
    proc2 = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="output2",
        stderr="",
    )
    event1 = Procs(proc=proc1)
    event2 = Procs(proc=proc2)
    assert event1 != event2


def test_equality_with_different_stderr():
    proc1 = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="",
        stderr="error1",
    )
    proc2 = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="",
        stderr="error2",
    )
    event1 = Procs(proc=proc1)
    event2 = Procs(proc=proc2)
    assert event1 != event2


def test_equality_with_dict():
    proc = subprocess.CompletedProcess(
        args=["echo", "test"],
        returncode=0,
        stdout="test",
        stderr="",
    )
    event = Procs(proc=proc)
    event_dict = event.to_dict()
    assert isinstance(event_dict, dict)


def test_equality_with_other_type():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event != 123
    assert event != ["list"]
    assert event != "string"


def test_not_equal_operator():
    proc1 = subprocess.CompletedProcess(
        args=["command1"],
        returncode=0,
    )
    proc2 = subprocess.CompletedProcess(
        args=["command2"],
        returncode=0,
    )
    event1 = Procs(proc=proc1)
    event2 = Procs(proc=proc2)
    assert event1 != event2


def test_context_includes_additional_fields():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="output",
        stderr="",
    )
    event = Procs(
        proc=proc,
        context={"user": "alice", "environment": "prod"},
    )
    assert event.context["user"] == "alice"
    assert event.context["environment"] == "prod"
    assert event.context["status"] == "succeed"
    assert event.context["command"] == "command"
    assert event.context["stdout"] == "output"
    assert event.context["stderr"] == ""


def test_to_dict():
    proc = subprocess.CompletedProcess(
        args=["test", "command"],
        returncode=0,
        stdout="output",
        stderr="",
    )
    event = Procs(proc=proc, id="proc123", name="test")
    result = event.to_dict()
    assert result["id"] == "proc123"
    assert result["name"] == "test"
    assert result["message"] == "succeed: test command"
    assert result["level"] == "INFO"
    assert result["code"] == 0
    assert result["context"]["status"] == "succeed"
    assert result["context"]["command"] == "test command"


def test_describe():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="output",
        stderr="",
    )
    event = Procs(proc=proc)
    result = event.describe()
    assert isinstance(result, dict)
    assert result["message"] == "succeed: command"
    assert result["level"] == "INFO"
    assert result["context"]["status"] == "succeed"


def test_inherits_from_base_event():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert hasattr(event, "ensure_serialisable")
    assert hasattr(event, "LEVELS")
    assert hasattr(event, "to_dict")
    assert hasattr(event, "describe")
    assert hasattr(event, "get")


def test_get_existing_attribute():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.get("level") == "INFO"
    assert event.get("ok") is True


def test_get_nonexistent_attribute():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.get("nonexistent") is None


def test_get_with_default():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
    )
    event = Procs(proc=proc)
    assert event.get("nonexistent", "default") == "default"


def test_proc_attribute_accessible():
    proc = subprocess.CompletedProcess(
        args=["command"],
        returncode=0,
        stdout="output",
    )
    event = Procs(proc=proc)
    assert event.proc is proc
    assert event.proc.args == ["command"]
    assert event.proc.returncode == 0


def test_multiple_procs_with_different_processes():
    proc1 = subprocess.CompletedProcess(args=["cmd1"], returncode=0)
    proc2 = subprocess.CompletedProcess(args=["cmd2"], returncode=1)
    proc3 = subprocess.CompletedProcess(args=["cmd3"], returncode=0)

    event1 = Procs(proc=proc1)
    event2 = Procs(proc=proc2)
    event3 = Procs(proc=proc3)

    assert event1.ok is True
    assert event2.ok is False
    assert event3.ok is True
    assert event1.id != event2.id
    assert event2.id != event3.id


def test_real_world_git_command_success():
    proc = subprocess.CompletedProcess(
        args=["git", "status"],
        returncode=0,
        stdout="On branch main\nnothing to commit, working tree clean\n",
        stderr="",
    )
    event = Procs(proc=proc)
    assert event.ok is True
    assert event.level == "INFO"
    assert event.message == "succeed: git status"
    assert "On branch main" in event.context["stdout"]


def test_real_world_command_failure():
    proc = subprocess.CompletedProcess(
        args=["ls", "/nonexistent/path"],
        returncode=1,
        stdout="",
        stderr="ls: cannot access '/nonexistent/path': No such file or directory",
    )
    event = Procs(proc=proc)
    assert event.ok is False
    assert event.level == "ERROR"
    assert event.message == "failed: ls /nonexistent/path"
    assert "No such file or directory" in event.context["stderr"]


def test_complex_command_with_args():
    proc = subprocess.CompletedProcess(
        args=["python", "-m", "pytest", "tests/", "-v", "--tb=short"],
        returncode=0,
        stdout="collected 10 items\nall tests passed",
        stderr="",
    )
    event = Procs(proc=proc)
    assert event.ok is True
    assert event.context["command"] == "python -m pytest tests/ -v --tb=short"
    assert "all tests passed" in event.context["stdout"]



from __future__ import annotations

from typing import Protocol, runtime_checkable

from xlog.event.base import EventLike


@runtime_checkable
class FormatLike(Protocol):
    """
    DESC:
        Protocol defining the interface for event formatters.
        Implementations convert EventLike objects to formatted strings.

    Params:
        event: EventLike, the event to format.

    Examples:
        ```python
        from xlog.format.base import FormatLike
        from xlog.event.logging import Log
        from xlog.format.json import Json

        # Use any formatter implementing FormatLike
        formatter: FormatLike = Json()
        event = Log(message="Test", level="INFO")
        output = formatter.format(event)
        ```
    """

    def format(
        self,
        event: EventLike,
    ) -> str:
        raise NotImplementedError(
            "Subclasses must implement the format method.",
        )



from .base.component import LogComponent
from .event.base import BaseEvent, EventLike
from .event.logging import Log
from .event.procs import Procs
from .format.base import FormatLike
from .format.colorjson import ColorJson
from .format.colortext import ColorText
from .format.colortree import ColorTree
from .format.json import Json
from .format.text import Text
from .format.tree import Tree
from .group.base import BaseGroup, GroupLike
from .group.filegroup import FileGroup
from .group.loggroup import LogGroup
from .node.base import NodeLike
from .node.logging import Logging
from .stream.stream import LogStream

__all__ = [
    "LogComponent",
    "BaseEvent",
    "EventLike",
    "Log",
    "Procs",
    "FormatLike",
    "Json",
    "Text",
    "Tree",
    "ColorJson",
    "ColorText",
    "ColorTree",
    "GroupLike",
    "BaseGroup",
    "LogGroup",
    "FileGroup",
    "NodeLike",
    "Logging",
    "LogStream",
]



from datetime import datetime

import pytz

from xlog.event.logging import Log
from xlog.format.colortree import ColorTree


def test_construct_with_defaults():
    formatter = ColorTree()
    assert formatter.width is None
    assert formatter.all_fields is False
    assert formatter.colors == ColorTree.COLORS
    assert formatter.prefix_timeformat == ColorTree.TIME_FORMAT
    assert formatter.prefix_format == ColorTree.FORMAT
    assert formatter.prefix_on is True


def test_construct_with_custom_width():
    formatter = ColorTree(width=120)
    assert formatter.width == 120
    assert formatter.all_fields is False


def test_construct_with_all_fields_true():
    formatter = ColorTree(all_fields=True)
    assert formatter.all_fields is True
    assert formatter.width is None


def test_construct_with_custom_colors():
    custom_colors = {
        "INFO": "blue",
        "ERROR": "magenta",
        "WARNING": "orange",
    }
    formatter = ColorTree(colors=custom_colors)
    assert formatter.colors == custom_colors


def test_construct_with_custom_prefix_timeformat():
    custom_timeformat = "%Y/%m/%d %H:%M"
    formatter = ColorTree(prefix_timeformat=custom_timeformat)
    assert formatter.prefix_timeformat == custom_timeformat
    assert formatter.prefix_format == ColorTree.FORMAT


def test_construct_with_custom_prefix_format():
    custom_format = "[{time}] {level} - {message}"
    formatter = ColorTree(prefix_format=custom_format)
    assert formatter.prefix_format == custom_format
    assert formatter.prefix_timeformat == ColorTree.TIME_FORMAT


def test_construct_with_prefix_off():
    formatter = ColorTree(prefix_on=False)
    assert formatter.prefix_on is False


def test_construct_with_all_custom_parameters():
    custom_colors = {"INFO": "blue", "ERROR": "red"}
    formatter = ColorTree(
        width=100,
        all_fields=True,
        colors=custom_colors,
        prefix_timeformat="%H:%M:%S",
        prefix_format="[{level}] {message}",
        prefix_on=False,
    )
    assert formatter.width == 100
    assert formatter.all_fields is True
    assert formatter.colors == custom_colors
    assert formatter.prefix_timeformat == "%H:%M:%S"
    assert formatter.prefix_format == "[{level}] {message}"
    assert formatter.prefix_on is False


def test_default_colors_mapping():
    assert ColorTree.COLORS["INFO"] == "green"
    assert ColorTree.COLORS["DEBUG"] == "cyan"
    assert ColorTree.COLORS["WARNING"] == "yellow"
    assert ColorTree.COLORS["ERROR"] == "red"
    assert ColorTree.COLORS["CRITICAL"] == "red"
    assert ColorTree.COLORS["UNKNOWN"] == "purple"


def test_resolve_timeformat_with_none_uses_default():
    formatter = ColorTree()
    result = formatter._resolve_timeformat(None)
    assert result == ColorTree.TIME_FORMAT


def test_resolve_timeformat_with_custom_value():
    formatter = ColorTree()
    custom = "%Y-%m-%d"
    result = formatter._resolve_timeformat(custom)
    assert result == custom


def test_resolve_format_with_none_uses_default():
    formatter = ColorTree()
    result = formatter._resolve_format(None)
    assert result == ColorTree.FORMAT


def test_resolve_format_with_custom_value():
    formatter = ColorTree()
    custom = "{time} | {level}"
    result = formatter._resolve_format(custom)
    assert result == custom


def test_resolve_prefix_with_none_uses_default():
    formatter = ColorTree()
    result = formatter._resolve_prefix(None)
    assert result is True


def test_resolve_prefix_with_false():
    formatter = ColorTree()
    result = formatter._resolve_prefix(False)
    assert result is False


def test_resolve_prefix_with_true():
    formatter = ColorTree()
    result = formatter._resolve_prefix(True)
    assert result is True


def test_pick_level_style_for_info():
    formatter = ColorTree()
    event = Log(message="Test", level="INFO", time=datetime.now(pytz.UTC))
    style = formatter._pick_level_style(event)
    assert style == "green"


def test_pick_level_style_for_debug():
    formatter = ColorTree()
    event = Log(message="Test", level="DEBUG", time=datetime.now(pytz.UTC))
    style = formatter._pick_level_style(event)
    assert style == "cyan"


def test_pick_level_style_for_warning():
    formatter = ColorTree()
    event = Log(message="Test", level="WARNING", time=datetime.now(pytz.UTC))
    style = formatter._pick_level_style(event)
    assert style == "yellow"


def test_pick_level_style_for_error():
    formatter = ColorTree()
    event = Log(message="Test", level="ERROR", time=datetime.now(pytz.UTC))
    style = formatter._pick_level_style(event)
    assert style == "red"


def test_pick_level_style_for_critical():
    formatter = ColorTree()
    event = Log(message="Test", level="CRITICAL", time=datetime.now(pytz.UTC))
    style = formatter._pick_level_style(event)
    assert style == "red"


def test_pick_level_style_for_unknown_level():
    formatter = ColorTree()
    event = Log(message="Test", level="CUSTOM", time=datetime.now(pytz.UTC))
    style = formatter._pick_level_style(event)
    # CUSTOM level is not valid, so it defaults to INFO which is green
    assert style == "green"


def test_pick_level_style_for_lowercase_level():
    formatter = ColorTree()
    event = Log(message="Test", level="info", time=datetime.now(pytz.UTC))
    style = formatter._pick_level_style(event)
    assert style == "green"


def test_pick_level_style_with_custom_colors():
    custom_colors = {"INFO": "blue", "ERROR": "purple"}
    formatter = ColorTree(colors=custom_colors)
    event = Log(message="Test", level="INFO", time=datetime.now(pytz.UTC))
    style = formatter._pick_level_style(event)
    assert style == "blue"


def test_pick_level_style_without_level_attribute():
    formatter = ColorTree()

    class MockEvent:
        message = "Test"
        time = datetime.now(pytz.UTC)

    event = MockEvent()
    style = formatter._pick_level_style(event)
    assert style == "green"


def test_format_basic_event():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert len(result) > 0
    assert "message: Test message" in result


def test_format_event_with_prefix_on():
    formatter = ColorTree(prefix_on=True)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "2025-12-06 10:30:45" in result
    assert "INFO" in result


def test_format_event_with_prefix_off():
    formatter = ColorTree(prefix_on=False)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "message: Test message" in result


def test_format_event_with_custom_prefix_format():
    formatter = ColorTree(prefix_format="[{level}] {time}")
    event = Log(
        message="Test message",
        level="WARNING",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "WARNING" in result
    assert "2025-12-06 10:30:45" in result


def test_format_event_with_custom_prefix_timeformat():
    formatter = ColorTree(prefix_timeformat="%Y/%m/%d")
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "2025/12/06" in result


def test_format_event_with_context():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        context={"job": "batch_process", "env": "production"},
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "context" in result


def test_format_event_with_tags():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        tags={"module": "formatter", "version": "2.0"},
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "tags" in result


def test_format_event_with_metrics():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        metrics={"duration": 150, "count": 10},
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "metrics" in result


def test_format_event_with_all_fields_false_filters_empty():
    formatter = ColorTree(all_fields=False)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        context=None,
        tags=None,
        metrics=None,
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    # Empty fields should not appear in output


def test_format_event_with_all_fields_true_includes_empty():
    formatter = ColorTree(all_fields=True)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        context=None,
        tags=None,
        metrics=None,
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    # Should include all fields even if empty


def test_format_event_with_nested_dict_context():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        context={
            "job": "batch_process",
            "details": {"step": "validation", "stage": "pre-processing"},
        },
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "context" in result


def test_format_event_with_list_in_payload():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        extra={"items": ["item1", "item2", "item3"]},
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "extra" in result


def test_format_event_with_list_of_dicts():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        extra={
            "records": [
                {"id": 1, "name": "Alice"},
                {"id": 2, "name": "Bob"},
            ]
        },
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "extra" in result


def test_format_event_with_different_levels():
    formatter = ColorTree()

    levels = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
    for level in levels:
        event = Log(
            message=f"{level} message",
            level=level,
            time=datetime(2025, 12, 6, 10, 30, 45),
        )
        result = formatter.format(event)
        assert isinstance(result, str)
        assert level in result


def test_format_event_with_debug_level_uses_cyan():
    formatter = ColorTree()
    event = Log(
        message="Debug message",
        level="DEBUG",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "DEBUG" in result


def test_format_event_with_error_level_uses_red():
    formatter = ColorTree()
    event = Log(
        message="Error message",
        level="ERROR",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "ERROR" in result


def test_format_event_with_timezone():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45, tzinfo=pytz.UTC),
        tz="UTC",
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "message: Test message" in result


def test_format_event_with_code():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="ERROR",
        time=datetime(2025, 12, 6, 10, 30, 45),
        code=500,
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "code" in result


def test_format_event_with_custom_id():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        id="custom-id-12345",
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "id" in result


def test_format_event_with_custom_name():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        name="CustomEventName",
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "customeventname" in result.lower()


def test_format_non_eventlike_returns_empty():
    formatter = ColorTree()
    result = formatter.format("not an event")
    assert result == ""


def test_format_none_returns_empty():
    formatter = ColorTree()
    result = formatter.format(None)
    assert result == ""


def test_add_node_with_simple_value():
    from rich.tree import Tree as RichTree

    formatter = ColorTree()
    tree = RichTree("root")
    formatter._add_node(tree, "key", "value")
    # Should not raise any exceptions
    assert tree is not None


def test_add_node_with_dict_value():
    from rich.tree import Tree as RichTree

    formatter = ColorTree()
    tree = RichTree("root")
    formatter._add_node(tree, "parent", {"child1": "value1", "child2": "value2"})
    # Should not raise any exceptions
    assert tree is not None


def test_add_node_with_list_value():
    from rich.tree import Tree as RichTree

    formatter = ColorTree()
    tree = RichTree("root")
    formatter._add_node(tree, "items", ["item1", "item2", "item3"])
    # Should not raise any exceptions
    assert tree is not None


def test_add_node_with_nested_list_of_dicts():
    from rich.tree import Tree as RichTree

    formatter = ColorTree()
    tree = RichTree("root")
    formatter._add_node(
        tree,
        "records",
        [{"id": 1, "name": "Alice"}, {"id": 2, "name": "Bob"}],
    )
    # Should not raise any exceptions
    assert tree is not None


def test_format_event_with_width_constraint():
    formatter = ColorTree(width=80)
    event = Log(
        message="Test message with a very long description that might need wrapping",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert len(result) > 0


def test_format_event_filters_none_values():
    formatter = ColorTree(all_fields=False)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        context={"key1": "value1", "key2": None},
    )
    result = formatter.format(event)
    assert isinstance(result, str)


def test_format_event_filters_empty_string():
    formatter = ColorTree(all_fields=False)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        context={"key1": "value1", "key2": ""},
    )
    result = formatter.format(event)
    assert isinstance(result, str)


def test_format_event_filters_empty_list():
    formatter = ColorTree(all_fields=False)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        extra={"items": []},
    )
    result = formatter.format(event)
    assert isinstance(result, str)


def test_format_event_filters_empty_dict():
    formatter = ColorTree(all_fields=False)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        context={},
    )
    result = formatter.format(event)
    assert isinstance(result, str)


def test_format_event_filters_false_values():
    formatter = ColorTree(all_fields=False)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        extra={"flag": False},
    )
    result = formatter.format(event)
    assert isinstance(result, str)


def test_format_multiple_events_sequentially():
    formatter = ColorTree()
    events = [
        Log(message=f"Message {i}", level="INFO", time=datetime(2025, 12, 6, 10, 30, i))
        for i in range(5)
    ]
    for event in events:
        result = formatter.format(event)
        assert isinstance(result, str)
        assert len(result) > 0


def test_format_event_with_unicode_characters():
    formatter = ColorTree()
    event = Log(
        message="测试消息 with émojis 🎉",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "测试消息" in result or len(result) > 0


def test_format_event_with_special_characters():
    formatter = ColorTree()
    event = Log(
        message="Test \"quotes\" and 'apostrophes' and <tags>",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert len(result) > 0


def test_default_timeformat_constant():
    assert ColorTree.TIME_FORMAT == "%Y-%m-%d %H:%M:%S"


def test_default_format_constant():
    assert ColorTree.FORMAT == "[{time}][{level}] {message}"


def test_format_preserves_tree_structure():
    formatter = ColorTree()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        context={"level1": {"level2": {"level3": "deep value"}}},
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "context" in result


def test_format_with_custom_colors_overrides_defaults():
    custom_colors = {
        "INFO": "white",
        "DEBUG": "light_blue",
        "WARNING": "orange",
        "ERROR": "dark_red",
        "CRITICAL": "bold_red",
    }
    formatter = ColorTree(colors=custom_colors)
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    style = formatter._pick_level_style(event)
    assert style == "white"


def test_format_event_with_complex_nested_structure():
    formatter = ColorTree()
    event = Log(
        message="Complex structure",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        context={
            "service": "api",
            "request": {
                "method": "POST",
                "endpoint": "/api/v1/users",
                "headers": {"Content-Type": "application/json"},
                "body": {"name": "Alice", "email": "alice@example.com"},
            },
            "response": {
                "status": 201,
                "data": {"id": 123, "created": True},
            },
        },
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "context" in result


def test_add_node_with_mixed_list_content():
    from rich.tree import Tree as RichTree

    formatter = ColorTree()
    tree = RichTree("root")
    formatter._add_node(
        tree,
        "mixed",
        ["string", 123, {"key": "value"}, ["nested", "list"]],
    )
    # Should not raise any exceptions
    assert tree is not None


def test_format_event_with_all_optional_fields():
    formatter = ColorTree()
    event = Log(
        message="Complete event",
        level="INFO",
        time=datetime(2025, 12, 6, 10, 30, 45),
        id="evt-123",
        name="TestEvent",
        code=200,
        context={"key": "value"},
        tags={"tag": "value"},
        metrics={"metric": 100},
        extra={"extra": "data"},
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert len(result) > 0



from __future__ import annotations

from typing import Any, Dict, Optional, Protocol, runtime_checkable


@runtime_checkable
class NodeLike(Protocol):
    """
    DESC:
        Protocol defining the interface for logging nodes.
        Implementations handle the actual output of formatted log messages.

    Params:
        level: str, log level for the message.
        message: str, formatted message to log.
        extra: Optional[Dict[str, Any]] = None, additional metadata.

    Examples:
        ```python
        from xlog.node.base import NodeLike
        from xlog.node.logging import Logging

        # Use any node implementing NodeLike
        node: NodeLike = Logging(name="app", level="INFO")
        node.log("INFO", "Application started")
        ```
    """

    def log(
        self,
        level: str,
        message: str,
        *,
        extra: Optional[Dict[str, Any]] = None,
    ) -> None: ...



from __future__ import annotations

import logging
from datetime import datetime
from typing import Dict, Optional

from colorlog import ColoredFormatter

from xlog.event.base import EventLike


class ColorText:
    """
    DESC:
        Colored text formatter using colorlog for terminal output.
        Applies colors based on log level for improved readability.

    Params:
        timeformat: Optional[str] = None, strftime format for timestamp.
        textformat: Optional[str] = None, format string with colorlog placeholders.
        colors: Optional[Dict[str, str]] = None, custom color mapping for log levels.

    Examples:
        ```python
        from xlog.format.colortext import ColorText
        from xlog.event.logging import Log

        # Create colored formatter
        formatter = ColorText()
        event = Log(message="Test", level="INFO")
        formatted = formatter.format(event)
        ```
    """

    TIME_FORMAT = "%Y-%m-%d %H:%M:%S"
    FORMAT = "%(log_color)s[%(asctime)s][%(levelname)s]%(reset)s %(message)s"
    COLORS = {
        "INFO": "green",
        "DEBUG": "cyan",
        "WARNING": "yellow",
        "ERROR": "red",
        "CRITICAL": "red",
        "UNKNOWN": "purple",
    }

    def __init__(
        self,
        timeformat: Optional[str] = None,
        textformat: Optional[str] = None,
        colors: Optional[Dict[str, str]] = None,
    ):
        self.timeformat = timeformat or self.TIME_FORMAT
        self.textformat = textformat or self.FORMAT
        self.colors = colors or self.COLORS
        self._formatter = ColoredFormatter(
            self.textformat,
            datefmt=self.timeformat,
            log_colors=self.colors,
        )

    def format(
        self,
        event: EventLike,
    ) -> str:
        if not isinstance(event, EventLike):
            return ""

        elevel = getattr(event, "level", "INFO").upper()
        elevelno = getattr(logging, elevel, logging.INFO)
        record = logging.LogRecord(
            name=event.name,
            level=elevelno,
            levelname=elevel,
            pathname="",
            msg=event.message,
            args=(),
            lineno=0,
            exc_info=None,
        )
        dt = getattr(event, "time", None)
        if isinstance(dt, datetime):
            record.created = dt.timestamp()
            record.msecs = dt.microsecond / 1000.0
        return self._formatter.format(record)



import time
from datetime import datetime

import pytz

from xlog.event.logging import Log
from xlog.group.base import BaseGroup


class MockGroup(BaseGroup):
    def __init__(self, *args, **kwargs):
        self.consumed_events = []
        self.dropped_events = []
        self.error_events = []
        self.closed = False
        super().__init__(*args, **kwargs)

    def _consume(self, stream: str, event):
        self.consumed_events.append((stream, event))

    def _on_drop(self, stream: str, event):
        self.dropped_events.append((stream, event))

    def _on_close(self):
        self.closed = True

    def _on_error(self, stream: str, event, error: Exception):
        self.error_events.append((stream, event, error))


# ---------- ASYNC MODE TESTS ----------


def test_async_init_with_defaults():
    group = MockGroup(async_=True)
    assert group.store is True
    assert group.max_len == 100_000
    assert group._queue.maxsize == 1000
    assert group._worker.is_alive()
    assert group._worker.daemon is True
    assert group._async is True
    assert len(group.id) == 10

    group.close(timeout=1.0)
    assert not group._worker.is_alive()


def test_async_init_with_custom_parameters():
    group = MockGroup(
        async_=True,
        id="test123",
        name="TestGroup",
        store=False,
        max_queue=500,
        max_len=1000,
    )
    assert group.id == "test123"
    assert group.name == "testgroup"
    assert group.store is False
    assert group.max_len == 1000
    assert group._queue.maxsize == 500
    assert group._async is True

    group.close(timeout=1.0)


def test_async_init_with_none_max_len():
    group = MockGroup(async_=True, max_len=None)
    assert group.max_len is None

    group.close(timeout=1.0)


def test_async_worker_thread_starts_automatically():
    group = MockGroup(async_=True)
    assert group._worker.is_alive()
    assert group._worker.name == f"xlog-group:{group.name}"

    group.close(timeout=1.0)


def test_async_alive_returns_true():
    group = MockGroup(async_=True)
    assert group.alive() is True

    group.close(timeout=1.0)


def test_async_alive_returns_false_after_close():
    group = MockGroup(async_=True)
    group.close(timeout=1.0)
    assert group.alive() is False


def test_async_receive_adds_event_to_queue():
    group = MockGroup(async_=True)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event)
    time.sleep(0.2)

    assert len(group.consumed_events) == 1
    assert group.consumed_events[0] == ("stream1", event)
    group.close(timeout=1.0)


def test_async_receive_multiple_events():
    group = MockGroup(async_=True)
    event1 = Log(
        message="Event 1",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    event2 = Log(
        message="Event 2",
        level="DEBUG",
        time=datetime.now(pytz.UTC),
    )
    event3 = Log(
        message="Event 3",
        level="ERROR",
        time=datetime.now(pytz.UTC),
    )

    group.receive("stream1", event1)
    group.receive("stream2", event2)
    group.receive("stream1", event3)

    group.flush()  # Wait for all events to be processed

    assert len(group.consumed_events) == 3
    assert ("stream1", event1) in group.consumed_events
    assert ("stream2", event2) in group.consumed_events
    assert ("stream1", event3) in group.consumed_events

    group.close(timeout=1.0)


def test_async_receive_drops_oldest_when_queue_full():
    class SlowMockGroup(MockGroup):
        def _consume(self, stream, event):
            time.sleep(0.1)
            super()._consume(stream, event)

    group = SlowMockGroup(async_=True, max_queue=3)

    for i in range(10):
        event = Log(
            message=f"Event {i}",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event)

    time.sleep(0.5)
    assert len(group.dropped_events) > 0

    group.close(timeout=5.0)


def test_async_receive_after_stop_drops_event():
    group = MockGroup(async_=True)
    group._stop.set()

    event = Log(
        message="Late event",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event)
    time.sleep(0.1)
    assert len(group.dropped_events) > 0

    group.close(timeout=1.0)


def test_async_store_true_saves_events():
    group = MockGroup(async_=True, store=True)
    event1 = Log(
        message="Event 1",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    event2 = Log(
        message="Event 2",
        level="DEBUG",
        time=datetime.now(pytz.UTC),
    )

    group.receive("stream1", event1)
    group.receive("stream1", event2)

    group.flush()

    assert "stream1" in group._events
    assert len(group._events["stream1"]) == 2

    group.close(timeout=1.0)


def test_async_store_false_does_not_save_events():
    group = MockGroup(async_=True, store=False)
    event1 = Log(
        message="Event 1",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    event2 = Log(
        message="Event 2",
        level="DEBUG",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event1)
    group.receive("stream1", event2)
    group.flush()
    assert len(group.consumed_events) == 2
    assert len(group._events.get("stream1", [])) == 0

    group.close(timeout=1.0)


def test_async_store_respects_max_len():
    group = MockGroup(async_=True, store=True, max_len=3)

    for i in range(5):
        event = Log(
            message=f"Event {i}",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event)

    group.flush()
    assert len(group._events["stream1"]) == 3

    group.close(timeout=1.0)


def test_async_flush_waits_for_queue_processing():
    group = MockGroup(async_=True)

    for i in range(10):
        event = Log(
            message=f"Event {i}",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event)

    group.flush()
    assert len(group.consumed_events) == 10
    assert group._queue.qsize() == 0

    group.close(timeout=1.0)


def test_async_close_stops_worker_thread():
    group = MockGroup(async_=True)
    assert group._worker.is_alive()

    group.close(timeout=1.0)
    assert not group._worker.is_alive()
    assert group.closed is True


def test_async_close_flushes_queue_first():
    group = MockGroup(async_=True)
    for i in range(5):
        event = Log(
            message=f"Event {i}",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event)

    group.close(timeout=1.0)
    assert len(group.consumed_events) == 5


def test_async_on_error_hook_called():
    class ErrorGroup(MockGroup):
        def _consume(self, stream, event):
            super()._consume(stream, event)
            if "error" in event.message:
                raise ValueError("Test error")

    group = ErrorGroup(async_=True)
    event_ok = Log(
        message="OK event",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    event_error = Log(
        message="error event",
        level="ERROR",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event_ok)
    group.receive("stream1", event_error)
    group.flush()
    assert len(group.error_events) > 0

    group.close(timeout=1.0)


# ---------- SYNC MODE TESTS ----------


def test_sync_init_with_defaults():
    group = MockGroup(async_=False)
    assert group.store is True
    assert group.max_len is None
    assert group._queue is None
    assert group._worker is None
    assert group._async is False
    assert len(group.id) == 10

    group.close()


def test_sync_init_with_custom_parameters():
    group = MockGroup(
        async_=False,
        id="test456",
        name="SyncGroup",
        store=False,
    )
    assert group.id == "test456"
    assert group.name == "syncgroup"
    assert group.store is False
    assert group._async is False
    assert group._queue is None
    assert group._worker is None

    group.close()


def test_sync_alive_returns_none():
    group = MockGroup(async_=False)
    assert group.alive() is None

    group.close()


def test_sync_receive_processes_immediately():
    group = MockGroup(async_=False)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event)
    assert len(group.consumed_events) == 1
    assert group.consumed_events[0] == ("stream1", event)

    group.close()


def test_sync_receive_multiple_events():
    group = MockGroup(async_=False)
    event1 = Log(
        message="Event 1",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    event2 = Log(
        message="Event 2",
        level="DEBUG",
        time=datetime.now(pytz.UTC),
    )
    event3 = Log(
        message="Event 3",
        level="ERROR",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event1)
    group.receive("stream2", event2)
    group.receive("stream1", event3)
    assert len(group.consumed_events) == 3
    assert ("stream1", event1) in group.consumed_events
    assert ("stream2", event2) in group.consumed_events
    assert ("stream1", event3) in group.consumed_events

    group.close()


def test_sync_store_true_saves_events():
    group = MockGroup(async_=False, store=True)
    event1 = Log(
        message="Event 1",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    event2 = Log(
        message="Event 2",
        level="DEBUG",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event1)
    group.receive("stream1", event2)
    assert "stream1" in group._events
    assert len(group._events["stream1"]) == 2

    group.close()


def test_sync_store_false_does_not_save_events():
    group = MockGroup(async_=False, store=False)
    event1 = Log(
        message="Event 1",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    event2 = Log(
        message="Event 2",
        level="DEBUG",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event1)
    group.receive("stream1", event2)
    assert len(group.consumed_events) == 2
    assert len(group._events.get("stream1", [])) == 0

    group.close()


def test_sync_flush_is_noop():
    group = MockGroup(async_=False)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event)
    group.flush()
    assert len(group.consumed_events) == 1

    group.close()


def test_sync_close_is_noop():
    group = MockGroup(async_=False)
    event = Log(
        message="Test",
        level="INFO",
        time=datetime.now(pytz.UTC),
    )
    group.receive("stream1", event)
    group.close(timeout=1.0)
    assert len(group.consumed_events) == 1

    group.close()


def test_sync_no_dropping_behavior():
    group = MockGroup(async_=False)
    for i in range(100):
        event = Log(
            message=f"Event {i}",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event)

    assert len(group.consumed_events) == 100
    assert len(group.dropped_events) == 0

    group.close()


def test_sync_store_with_max_len():
    group = MockGroup(async_=False, store=True, max_len=3)
    for i in range(5):
        event = Log(
            message=f"Event {i}",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event)

    assert len(group._events["stream1"]) == 3
    group.close()


# ---------- COMMON TESTS (work for both async and sync) ----------


def test_list_streams_empty():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode)
        assert group.list_streams() == []
        group.close(timeout=1.0)


def test_list_streams_with_events():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode, store=True)
        event1 = Log(
            message="Event 1",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        event2 = Log(
            message="Event 2",
            level="DEBUG",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event1)
        group.receive("stream2", event2)
        group.receive("stream3", event1)
        if async_mode:
            group.flush()

        streams = group.list_streams()
        assert len(streams) == 3
        assert "stream1" in streams
        assert "stream2" in streams
        assert "stream3" in streams
        group.close(timeout=1.0)


def test_list_events_empty():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode)
        assert group.list_events() == []
        group.close(timeout=1.0)


def test_list_events_with_stored_events():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode, store=True)
        event1 = Log(
            message="Event 1",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        event2 = Log(
            message="Event 2",
            level="DEBUG",
            time=datetime.now(pytz.UTC),
        )
        event3 = Log(
            message="Event 3",
            level="ERROR",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event1)
        group.receive("stream2", event2)
        group.receive("stream1", event3)
        if async_mode:
            group.flush()

        events = group.list_events()
        assert len(events) == 3
        assert event1 in events
        assert event2 in events
        assert event3 in events
        group.close(timeout=1.0)


def test_list_events_by_stream():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode, store=True)
        event1 = Log(
            message="Stream1 Event",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        event2 = Log(
            message="Stream2 Event",
            level="DEBUG",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event1)
        group.receive("stream2", event2)

        if async_mode:
            group.flush()

        stream1_events = group.list_events("stream1")
        assert len(stream1_events) == 1
        assert event1 in stream1_events

        stream2_events = group.list_events("stream2")
        assert len(stream2_events) == 1
        assert event2 in stream2_events

        group.close(timeout=1.0)


def test_describe_returns_dict():
    for async_mode in [True, False]:
        group = MockGroup(
            async_=async_mode,
            id="test123",
            name="TestGroup",
            store=True,
        )
        result = group.describe()
        assert isinstance(result, dict)
        assert result["id"] == "test123"
        assert result["name"] == "testgroup"
        assert result["store"] is True
        assert "streams" in result
        assert "queue_size" in result
        assert "alive" in result
        if async_mode:
            assert result["alive"] is True
            assert result["max_queue"] > 0
        else:
            assert result["alive"] is None
            assert result["max_queue"] == 0

        group.close(timeout=1.0)


def test_to_dict_returns_dict():
    for async_mode in [True, False]:
        group = MockGroup(
            async_=async_mode,
            id="test123",
            name="TestGroup",
            store=True,
        )
        result = group.to_dict()
        assert isinstance(result, dict)
        assert result["id"] == "test123"
        assert result["name"] == "testgroup"
        assert result["store"] is True
        assert "streams" in result
        assert isinstance(result["streams"], dict)

        group.close(timeout=1.0)


def test_to_dict_includes_event_data():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode, store=True)
        event = Log(
            message="Test message",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event)

        if async_mode:
            group.flush()

        result = group.to_dict()
        assert "stream1" in result["streams"]
        assert len(result["streams"]["stream1"]) == 1
        assert result["streams"]["stream1"][0]["message"] == "Test message"

        group.close(timeout=1.0)


def test_multiple_streams_independent():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode, store=True)
        event1 = Log(
            message="Stream1 Event",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        event2 = Log(
            message="Stream2 Event",
            level="DEBUG",
            time=datetime.now(pytz.UTC),
        )
        group.receive("stream1", event1)
        group.receive("stream2", event2)
        if async_mode:
            group.flush()
        assert len(group._events["stream1"]) == 1
        assert len(group._events["stream2"]) == 1
        assert group._events["stream1"][0].message == "Stream1 Event"
        assert group._events["stream2"][0].message == "Stream2 Event"

        group.close(timeout=1.0)


def test_event_ordering_preserved():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode, store=True)
        events = []
        for i in range(5):
            event = Log(
                message=f"Event {i}",
                level="INFO",
                time=datetime.now(pytz.UTC),
            )
            events.append(event)
            group.receive("stream1", event)

        if async_mode:
            group.flush()

        stored = list(group._events["stream1"])
        for i, event in enumerate(events):
            assert stored[i].message == f"Event {i}"

        group.close(timeout=1.0)


def test_inherits_from_log_component():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode)
        assert hasattr(group, "id")
        assert hasattr(group, "name")
        group.close(timeout=1.0)


def test_store_converts_to_bool():
    for async_mode in [True, False]:
        group1 = MockGroup(async_=async_mode, store=1)
        assert group1.store is True

        group1.close(timeout=1.0)
        group2 = MockGroup(async_=async_mode, store=0)
        assert group2.store is False

        group2.close(timeout=1.0)
        group3 = MockGroup(async_=async_mode, store="yes")
        assert group3.store is True
        group3.close(timeout=1.0)


def test_async_converts_to_bool():
    group1 = MockGroup(async_=1)
    assert group1._async is True

    group1.close(timeout=1.0)
    group2 = MockGroup(async_=0)
    assert group2._async is False

    group2.close(timeout=1.0)
    group3 = MockGroup(async_="yes")
    assert group3._async is True

    group3.close(timeout=1.0)


def test_defaultdict_creates_new_streams():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode, store=True)
        event = Log(
            message="Test",
            level="INFO",
            time=datetime.now(pytz.UTC),
        )
        assert "new_stream" not in group._events

        group.receive("new_stream", event)
        if async_mode:
            group.flush()
        assert "new_stream" in group._events

        group.close(timeout=1.0)


def test_deque_maxlen_behavior():
    for async_mode in [True, False]:
        group = MockGroup(async_=async_mode, store=True, max_len=3)

        for i in range(5):
            event = Log(
                message=f"Event {i}",
                level="INFO",
                time=datetime.now(pytz.UTC),
            )
            group.receive("stream1", event)

        if async_mode:
            group.flush()

        stored = list(group._events["stream1"])
        assert len(stored) == 3
        assert stored[0].message == "Event 2"
        assert stored[1].message == "Event 3"
        assert stored[2].message == "Event 4"

        group.close(timeout=1.0)


def test_consume_must_be_implemented():
    assert hasattr(BaseGroup, "_consume")



import logging
from datetime import datetime

import pytz

from xlog.event.logging import Log


def test_init_with_minimal_args():
    event = Log(message="Test message")
    assert event.message == "Test message"
    assert event.level == "INFO"
    assert event.levelno == logging.INFO
    assert event.tz is None
    assert event.code is None
    assert event.context == {}
    assert event.tags == {}
    assert event.metrics == {}
    assert event.extra == {}
    assert len(event.id) == 10
    assert len(event.identifier) == 12


def test_init_with_all_args():
    now = datetime(2023, 11, 23, 15, 30, 45, tzinfo=pytz.UTC)
    event = Log(
        message="Full event",
        id="event123",
        name="TestEvent",
        time=now,
        tz=pytz.UTC,
        level=logging.WARNING,
        code=404,
        context={"user": "alice"},
        tags={"env": "prod"},
        metrics={"duration": "100ms"},
        extra={"custom": "data"},
    )
    assert event.message == "Full event"
    assert event.id == "event123"
    assert event.name == "testevent"
    assert event.time.isoformat() == now.isoformat()
    assert event.tz == pytz.UTC
    assert event.level == "WARNING"
    assert event.levelno == logging.WARNING
    assert event.code == 404
    assert event.context == {"user": "alice"}
    assert event.tags == {"env": "prod"}
    assert event.metrics == {"duration": "100ms"}
    assert event.extra == {"custom": "data"}


def test_message_is_stripped():
    event = Log(message="  Test message  \n")
    assert event.message == "Test message"


def test_empty_message():
    event = Log(message="")
    assert event.message == ""


def test_none_message():
    event = Log(message=None)
    assert event.message == ""


def test_level_defaults_to_info():
    event = Log(message="Test")
    assert event.level == "INFO"
    assert event.levelno == logging.INFO


def test_level_is_uppercase():
    event = Log(message="Test", level="debug")
    assert event.level == "DEBUG"
    assert event.levelno == logging.DEBUG


def test_invalid_level_defaults_to_info():
    event = Log(message="Test", level="INVALID_LEVEL")
    assert event.level == "INFO"
    assert event.levelno == logging.INFO


def test_valid_levels():
    for level in ["DEBUG", "INFO", "WARNING", "ERROR", "UNKNOWN"]:
        event = Log(message="Test", level=level)
        assert event.level == level
        assert event.levelno == getattr(logging, level, logging.INFO)


def test_code_as_integer():
    event = Log(message="Test", code=200)
    assert event.code == 200
    assert isinstance(event.code, int)


def test_code_as_string_converts_to_int():
    event = Log(message="Test", code="404")
    assert event.code == 404
    assert isinstance(event.code, int)


def test_code_invalid_string_returns_none():
    event = Log(message="Test", code="invalid")
    assert event.code is None


def test_code_none():
    event = Log(message="Test", code=None)
    assert event.code is None


def test_tz_as_string():
    event = Log(message="Test", tz="Europe/London")
    assert event.tz == pytz.timezone("Europe/London")


def test_tz_as_pytz_object():
    tz = pytz.timezone("Asia/Tokyo")
    event = Log(message="Test", tz=tz)
    assert event.tz == pytz.timezone("Asia/Tokyo")


def test_tz_invalid_defaults_to_utc():
    event = Log(message="Test", tz="Invalid/Timezone")
    assert event.tz is None
    assert event.time.tzinfo == pytz.UTC


def test_tz_none_defaults_to_utc():
    event = Log(message="Test", tz=None)
    assert event.tz is None
    assert event.time.tzinfo == pytz.UTC


def test_time_as_datetime():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    event = Log(message="Test", time=now)
    assert event.time == now


def test_time_as_iso_string():
    time_str = "2023-11-23T12:00:00"
    event = Log(message="Test", time=time_str)
    expected = datetime.fromisoformat(time_str)
    assert event.time.replace(tzinfo=None) == expected


def test_time_invalid_string_uses_now():
    event = Log(message="Test", time="invalid")
    assert isinstance(event.time, datetime)


def test_time_none_uses_now():
    event = Log(message="Test", time=None)
    assert isinstance(event.time, datetime)


def test_context_serialization():
    event = Log(
        message="Test",
        context={"timestamp": datetime(2023, 11, 23, 10, 0, 0)},
    )
    assert event.context["timestamp"] == "2023-11-23T10:00:00"


def test_tags_serialization():
    event = Log(
        message="Test",
        tags={"created": datetime(2023, 11, 23, 10, 0, 0)},
    )
    assert event.tags["created"] == "2023-11-23T10:00:00"


def test_metrics_serialization():
    event = Log(
        message="Test",
        metrics={"logged": datetime(2023, 11, 23, 10, 0, 0)},
    )
    assert event.metrics["logged"] == "2023-11-23T10:00:00"


def test_extra_serialization():
    event = Log(
        message="Test",
        extra={"recorded": datetime(2023, 11, 23, 10, 0, 0)},
    )
    assert event.extra["recorded"] == "2023-11-23T10:00:00"


def test_identifier_is_consistent():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    event1 = Log(
        message="Test",
        name="test",
        time=now,
        level="INFO",
        code=200,
    )
    event2 = Log(
        message="Test",
        name="test",
        time=now,
        level="INFO",
        code=200,
    )
    assert event1.identifier == event2.identifier


def test_identifier_changes_with_different_data():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    event1 = Log(message="Test 1", time=now)
    event2 = Log(message="Test 2", time=now)
    assert event1.identifier != event2.identifier


def test_equality_with_same_event():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    event1 = Log(message="Test", name="test", time=now)
    event2 = Log(message="Test", name="test", time=now)
    assert event1 == event2


def test_equality_with_different_event():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    event1 = Log(message="Test 1", time=now)
    event2 = Log(message="Test 2", time=now)
    assert event1 != event2


def test_equality_with_dict():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    event = Log(message="Test", name="test", time=now)
    event_dict = event.to_dict()
    assert event == event_dict


def test_equality_with_identifier_string():
    event = Log(message="Test")
    assert event == event.identifier


def test_equality_with_other_type():
    event = Log(message="Test")
    assert event != 123
    assert event != ["list"]
    assert event is not None


def test_not_equal_operator():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    event1 = Log(message="Test 1", time=now)
    event2 = Log(message="Test 2", time=now)
    assert event1 != event2


def test_get_existing_attribute():
    event = Log(message="Test", level="WARNING")
    assert event.get("level") == "WARNING"
    assert event.get("levelno") == logging.WARNING
    assert event.get("message") == "Test"


def test_get_nonexistent_attribute():
    event = Log(message="Test")
    assert event.get("nonexistent") is None


def test_get_with_default():
    event = Log(message="Test")
    assert event.get("nonexistent", "default") == "default"


def test_to_dict():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    event = Log(
        message="Test",
        id="event123",
        name="test",
        time=now,
        tz=pytz.UTC,
        level=logging.INFO,
        code=200,
        context={"key": "value"},
        tags={"tag": "val"},
        metrics={"metric": "val"},
        extra={"extra": "val"},
    )
    result = event.to_dict()
    assert result["id"] == "event123"
    assert result["name"] == "test"
    assert result["message"] == "Test"
    assert result["time"] == now.isoformat()
    assert result["tz"] == "UTC"
    assert result["level"] == "INFO"
    assert result["levelno"] == logging.INFO
    assert result["code"] == 200
    assert result["context"] == {"key": "value"}
    assert result["tags"] == {"tag": "val"}
    assert result["metrics"] == {"metric": "val"}
    assert result["extra"] == {"extra": "val"}


def test_describe():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    event = Log(message="Test", time=now)
    result = event.describe()
    assert isinstance(result, dict)
    assert result["message"] == "Test"
    assert result["time"] == "2023-11-23T12:00:00+00:00"
    assert result["level"] == "INFO"


def test_from_dict_with_dict():
    data = {
        "id": "event123",
        "message": "Test message",
        "name": "test",
        "level": logging.ERROR,
        "code": 500,
        "context": {"user": "bob"},
    }
    event = Log.from_dict(data)
    assert event.id == "event123"
    assert event.message == "Test message"
    assert event.name == "test"
    assert event.level == "ERROR"
    assert event.levelno == logging.ERROR
    assert event.code == 500
    assert event.context == {"user": "bob"}


def test_from_dict_with_event_object():
    original = Log(message="Test", level="DEBUG")
    result = Log.from_dict(original)
    assert result is original


def test_from_dict_with_time_string():
    data = {
        "message": "Test",
        "time": "2023-11-23T12:00:00",
    }
    event = Log.from_dict(data)
    expected = datetime.fromisoformat("2023-11-23T12:00:00")
    assert event.time.replace(tzinfo=None) == expected


def test_from_dict_with_missing_fields():
    data = {"message": "Test"}
    event = Log.from_dict(data)
    assert event.message == "Test"
    assert event.level == "INFO"
    assert event.levelno == logging.INFO


def test_from_dict_round_trip():
    now = datetime(2023, 11, 23, 12, 0, 0, tzinfo=pytz.UTC)
    original = Log(
        message="Test",
        name="test",
        time=now,
        level="WARNING",
        code=404,
        context={"key": "value"},
    )
    data = original.to_dict()
    recreated = Log.from_dict(data)
    assert original == recreated


def test_inherits_from_base_event():
    event = Log(message="Test")
    assert hasattr(event, "ensure_serialisable")
    assert hasattr(event, "LEVELS")
    assert hasattr(event, "_resolve_level")
    assert hasattr(event, "_resolve_message")


def test_str_representation():
    event = Log(message="This is my message")
    assert str(event) == "This is my message"


def test_str_with_empty_message():
    event = Log(message="")
    assert str(event) == ""


def test_repr_representation():
    event = Log(message="Test message", level="ERROR")
    result = repr(event)
    assert "Log" in result
    assert "message=Test message" in result
    assert "level=ERROR" in result


def test_repr_with_info_level():
    event = Log(message="Info message")
    result = repr(event)
    assert "Log" in result
    assert "message=Info message" in result
    assert "level=INFO" in result


def test_kwargs_are_passed_to_parent():
    event = Log(message="Test", custom_kwarg="value")
    assert event.message == "Test"


def test_multiple_events_have_unique_ids():
    event1 = Log(message="Event 1")
    event2 = Log(message="Event 2")
    event3 = Log(message="Event 3")

    assert event1.id != event2.id
    assert event2.id != event3.id
    assert event1.id != event3.id


def test_multiple_events_have_unique_identifiers():
    event1 = Log(message="Event 1")
    event2 = Log(message="Event 2")
    event3 = Log(message="Event 3")

    assert event1.identifier != event2.identifier
    assert event2.identifier != event3.identifier
    assert event1.identifier != event3.identifier


def test_log_event_with_context():
    event = Log(
        message="User login",
        level=logging.INFO,
        context={
            "user_id": "12345",
            "ip_address": "192.168.1.1",
            "timestamp": datetime(2023, 11, 23, 10, 0, 0),
        },
    )
    assert event.message == "User login"
    assert event.context["user_id"] == "12345"
    assert event.context["ip_address"] == "192.168.1.1"
    assert event.context["timestamp"] == "2023-11-23T10:00:00"


def test_log_event_with_tags():
    event = Log(
        message="Database query",
        tags={
            "service": "api",
            "environment": "production",
            "version": "1.2.3",
        },
    )
    assert event.tags["service"] == "api"
    assert event.tags["environment"] == "production"
    assert event.tags["version"] == "1.2.3"


def test_log_event_with_metrics():
    event = Log(
        message="Request completed",
        metrics={
            "duration_ms": "150",
            "memory_mb": "256",
            "cpu_percent": "45.5",
        },
    )
    assert event.metrics["duration_ms"] == "150"
    assert event.metrics["memory_mb"] == "256"
    assert event.metrics["cpu_percent"] == "45.5"


def test_log_event_error_with_code():
    event = Log(
        message="Database connection failed",
        level="ERROR",
        code=500,
        context={"error": "Connection timeout"},
    )
    assert event.message == "Database connection failed"
    assert event.level == "ERROR"
    assert event.code == 500
    assert event.context["error"] == "Connection timeout"


def test_log_event_debug_message():
    event = Log(
        message="Debug information",
        level="DEBUG",
        extra={"stack_trace": "line 1\nline 2\nline 3"},
    )
    assert event.level == "DEBUG"
    assert event.levelno == logging.DEBUG
    assert event.message == "Debug information"
    assert event.extra["stack_trace"] == "line 1\nline 2\nline 3"



from datetime import datetime

from xlog.event.logging import Log
from xlog.format.colortext import ColorText


def test_construct_with_defaults():
    formatter = ColorText()
    assert formatter._formatter is not None


def test_construct_with_custom_timeformat():
    custom_time = "%Y/%m/%d %H:%M"
    formatter = ColorText(timeformat=custom_time)
    assert formatter._formatter is not None


def test_construct_with_custom_format():
    custom_format = "%(log_color)s%(levelname)s%(reset)s - %(message)s"
    formatter = ColorText(textformat=custom_format)
    assert formatter._formatter is not None


def test_construct_with_custom_colors():
    custom_colors = {
        "INFO": "blue",
        "DEBUG": "white",
        "WARNING": "orange",
        "ERROR": "red",
        "UNKNOWN": "purple",
    }
    formatter = ColorText(colors=custom_colors)
    assert formatter._formatter is not None


def test_construct_with_all_custom_parameters():
    custom_time = "%H:%M:%S"
    custom_format = "%(log_color)s[%(asctime)s]%(reset)s %(message)s"
    custom_colors = {"INFO": "cyan", "ERROR": "magenta"}
    formatter = ColorText(
        timeformat=custom_time,
        textformat=custom_format,
        colors=custom_colors,
    )
    assert formatter._formatter is not None


def test_format_basic_info_event():
    formatter = ColorText()
    event = Log(
        message="Info message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "Info message" in result
    assert len(result) > len("Info message")


def test_format_debug_event():
    formatter = ColorText()
    event = Log(
        message="Debug message",
        level="DEBUG",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "Debug message" in result


def test_format_warning_event():
    formatter = ColorText()
    event = Log(
        message="Warning message",
        level="WARNING",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "Warning message" in result


def test_format_error_event():
    formatter = ColorText()
    event = Log(
        message="Error message",
        level="ERROR",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "Error message" in result


def test_format_unknown_event():
    formatter = ColorText()
    event = Log(
        message="Unknown level message",
        level="UNKNOWN",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert "Unknown level message" in result


def test_format_event_contains_level():
    formatter = ColorText()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "INFO" in result


def test_format_event_contains_timestamp():
    formatter = ColorText()
    event = Log(
        message="Test message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "2025" in result or "10:30" in result


def test_format_non_Log_returns_empty_string():
    formatter = ColorText()
    result = formatter.format("not a log event")
    assert result == ""

    result = formatter.format(None)
    assert result == ""

    result = formatter.format(123)
    assert result == ""

    result = formatter.format({"message": "dict"})
    assert result == ""


def test_format_event_with_empty_message():
    formatter = ColorText()
    event = Log(
        message="",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "INFO" in result


def test_format_event_with_multiline_message():
    formatter = ColorText()
    event = Log(
        message="Line 1\nLine 2\nLine 3",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "Line 1" in result
    assert "Line 2" in result
    assert "Line 3" in result


def test_format_event_with_special_characters():
    formatter = ColorText()
    event = Log(
        message="Special chars: @#$%^&*(){}[]",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "Special chars: @#$%^&*(){}[]" in result


def test_format_event_with_unicode_characters():
    formatter = ColorText()
    event = Log(
        message="Unicode test: 你好世界 🚀 café",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "Unicode test: 你好世界 🚀 café" in result


def test_format_event_with_long_message():
    formatter = ColorText()
    long_message = "x" * 1000
    event = Log(
        message=long_message,
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert long_message in result


def test_format_multiple_events_consecutively():
    formatter = ColorText()

    event1 = Log(
        message="First event",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    event2 = Log(
        message="Second event",
        level="ERROR",
        time=datetime(2025, 11, 22, 10, 31, 00),
    )
    event3 = Log(
        message="Third event",
        level="DEBUG",
        time=datetime(2025, 11, 22, 10, 31, 15),
    )

    result1 = formatter.format(event1)
    result2 = formatter.format(event2)
    result3 = formatter.format(event3)

    assert "First event" in result1
    assert "INFO" in result1
    assert "Second event" in result2
    assert "ERROR" in result2
    assert "Third event" in result3
    assert "DEBUG" in result3


def test_format_different_events_have_different_colors():
    formatter = ColorText()
    event_info = Log(
        message="Same message",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    event_error = Log(
        message="Same message",
        level="ERROR",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )

    result_info = formatter.format(event_info)
    result_error = formatter.format(event_error)

    assert result_info != result_error
    assert "Same message" in result_info
    assert "Same message" in result_error
    assert "INFO" in result_info
    assert "ERROR" in result_error


def test_format_preserves_event_name_in_fake_record():
    formatter = ColorText()
    event = Log(
        message="Test message",
        name="TestLogger",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert isinstance(result, str)
    assert len(result) > 0


def test_format_event_with_custom_colors_produces_output():
    custom_colors = {
        "INFO": "cyan",
        "DEBUG": "white",
        "WARNING": "yellow",
        "ERROR": "red",
        "UNKNOWN": "magenta",
    }
    formatter = ColorText(colors=custom_colors)
    event = Log(
        message="Custom color test",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
    )
    result = formatter.format(event)
    assert "Custom color test" in result
    assert "INFO" in result


def test_default_constants_are_set():
    assert ColorText.TIME_FORMAT == "%Y-%m-%d %H:%M:%S"
    assert "%(log_color)s" in ColorText.FORMAT
    assert "%(levelname)s" in ColorText.FORMAT
    assert "%(reset)s" in ColorText.FORMAT
    assert "%(message)s" in ColorText.FORMAT
    assert ColorText.COLORS["INFO"] == "green"
    assert ColorText.COLORS["DEBUG"] == "cyan"
    assert ColorText.COLORS["WARNING"] == "yellow"
    assert ColorText.COLORS["ERROR"] == "red"
    assert ColorText.COLORS["UNKNOWN"] == "purple"


def test_format_event_with_context_tags_metrics():
    formatter = ColorText()
    event = Log(
        message="Event with metadata",
        level="INFO",
        time=datetime(2025, 11, 22, 10, 30, 45),
        context={"job": "test"},
        tags={"module": "formatter"},
        metrics={"duration": "10ms"},
    )
    result = formatter.format(event)
    assert "Event with metadata" in result
    assert "INFO" in result
